require 'active_support/core_ext/string'

module Bob
  module Compiler
    class Buffer
      class SameLineProxy
        def initialize(buffer)
          @buffer = buffer
        end

        def <<(string)
          @buffer.<< string, true
          self
        end
      end

      attr_reader :string
      alias_method :to_s, :string

      def initialize(indent_level = 0)
        @string = ''
        @indent_level = indent_level
        @same_line_proxy = SameLineProxy.new(self)
      end

      def <<(string, same_line = false)
        newline! unless first_line? || same_line

        if indented?
          string = indent(string, @indent_level)
        end

        if same_line
          string = string.lstrip
        end

        @string << string

        @same_line_proxy
      end

      def merge(buffer)
        buffer.string.each_line.with_index do |line, i|
          self.<<(line.chomp, i == 0)
        end

        @same_line_proxy
      end

      def newline!
        @string << "\n"
      end

      alias_method :blankline!, :newline!

      def indented(increment = 2)
        indent_level_was = @indent_level
        @indent_level += increment
        yield
      ensure
        @indent_level = indent_level_was
      end

      private
        def first_line?
          @string.empty?
        end

        def indented?
          @indent_level > 0
        end

        def indent(str, multiplier = 2)
          spaces = " " * multiplier
          str.each_line.map { |line| line.blank? ? line : "#{spaces}#{line}" }.join
        end
    end
  end
end
