require "bob/compiler/walker"
require "bob/compiler/editable"

module Bob
  module Compiler
    class Layout < Walker
      include Editable

      attr_reader :container

      private
        def compile
          super

          if container.nil?
            error("No container defined", @node.children.try(:last) || @node)
          end
        end

        def after_element(node, root)
          if is_container?(node)
            if container
              warn "Multiple containers found, extra contains are ignored", node
            else
              assign_container(node)
            end
          end

          super
        end

        def filtered_children(node)
          is_container?(node) ? [] : super
        end

        def is_container?(node)
          node.has_attribute?("bob-container")
        end

        def assign_container(node)
          @container = node

          @buffer << <<-JS.strip_heredoc.chomp
            descriptor.container = {
              element: dom.getElement()
            };
          JS
        end
    end
  end
end
