module Bob
  module Compiler
    module Substitution
      SUBSTITUTION_SCANNER = /(\\?{{.+?}})/

      SUBSTITUTION_PARSER  = /
        \A
          {{
            [[:blank:]]*
            (?<path> [0-9a-z_]+ (?:\.[0-9a-z_]+)* )
            [[:blank:]]*
            (?:
              \|\|
              [[:blank:]]*
              (?:
                (?: ' (?<fallback> (?:[^'\\{}]|\\.)*) ' ) |
                (?: " (?<fallback> (?:[^"\\{}]|\\.)*) " )
              )
              [[:blank:]]*
            )?
          }}
        \z
      /xi

      def quote_and_substitute(arg, node)
        parts = arg.split(SUBSTITUTION_SCANNER).reject(&:empty?)

        if parts.empty?
          %{""}
        else
          parts.map { |str|

            if SUBSTITUTION_SCANNER =~ str
              if str[0] == "\\"
                quote(str[1..-1] || "")
              else
                if (parsed = SUBSTITUTION_PARSER.match(str))
                  if parsed[:fallback]
                    "env.get(#{quote parsed[:path]}, #{quote unescape parsed[:fallback]})"
                  else
                    "env.get(#{quote parsed[:path]})"
                  end
                else
                  warn "Treating invalid substitution as string: #{str.inspect}", node.parent.line
                  quote str
                end
              end
            else
              quote str
            end

          }.join(" + ")
        end
      end

      def unescape(str)
        str.gsub(/\\(.)/, "\\1")
      end
    end
  end
end