require 'bob/compiler/base'
require 'bob/compiler/buffer'
require 'bob/compiler/editable'
require 'bob/compiler/substitution'

module Bob
  module Compiler
    class Walker < Base
      include Substitution

      def initialize(node)
        super()
        @node = node
      end

      private
        def compile
          @buffer << "function(dom, env) {"

          @buffer.indented do
            @buffer << "var descriptor = #{pretty_quote descriptor};"

            @buffer.blankline!

            walk(@node, true)

            @buffer.blankline!

            @buffer << "descriptor.element = dom.getElement();"

            @buffer.blankline!

            @buffer << "return descriptor;"
          end

          @buffer << "}"
        end

        def descriptor
          {
            element: nil,
            editables: []
          }
        end

        def walk(node, root = false)
          if node.element?
            visit_element(node, root)
          elsif node.text? || node.cdata?
            visit_text_node(node)
          elsif node.comment?
            visit_comment_node(node)
          else
            raise TypeError, "Unsupported node type #{node.type}"
          end

          if node.element?
            filtered_children(node).each do |child|
              walk(child)
            end

            after_element(node, root)
          end
        end

        def filtered_children(node)
          node.children.select { |child| child.element? || child.text? || child.cdata? || child.comment? }
        end

        def filtered_attributes(node)
          # TODO: switch to a whitelist
          node.attribute_nodes.reject { |attr| attr.name.start_with?('bob-') }
        end

        def visit_element(node, root)
          build_element(node, root)
        end

        def after_element(node, root)
          append_element(node) unless root
        end

        def visit_text_node(node)
          build_text_node(node)
        end

        def visit_comment_node(node)
          build_comment_node(node)
        end

        def build_element(node, root)
          @buffer << "dom.createElement(#{quote node.node_name});"

          filtered_attributes(node).each do |attr|
            @buffer << "dom.setAttribute(#{quote attr.name}, #{quote_and_substitute attr.value, node});"
          end
        end

        def append_element(node)
          @buffer << "dom.appendElement();"
        end

        def build_text_node(node)
          @buffer << "dom.appendTextNode(#{quote_and_substitute node.text, node});"
        end

        def build_comment_node(node)
          @buffer << "dom.appendComment(#{quote_and_substitute node.text, node});"
        end
    end
  end
end
